@@ -6,16 +6,14 @@
 
 from .__info__ import __author__, __copyright__, __license__, __version__
 
-from .handlers import *
+from .features import *
 from .helpers import *
-from .loglib import *
 from .parser import *
 from .preimports import *
 from .warnings import *
 
-from .handlers import __features__ as _handlers
+from .features import __features__ as _features
 from .helpers import __features__ as _helpers
-from .loglib import __features__ as _loglib
 from .parser import __features__ as _parser
 from .preimports import __features__ as _preimports
 from .warnings import __features__ as _warnings
@@ -27,4 +25,5 @@ ts.__license__   = __license__
 ts.__version__   = __version__
 
 
-__all__ = _handlers + _helpers + _loglib + _parser + _preimports + _warnings
+__all__ = _features + _helpers + _parser + _preimports + _warnings
+
