@@ -3,16 +3,18 @@
 
 """
 from .common import *
+from .config import *
 from .files import *
 from .hash import *
 from .network import *
 from .strings import *
 
 from .common import __features__ as _common
+from .config import __features__ as _config
 from .files import __features__ as _files
 from .hash import __features__ as _hash
 from .network import __features__ as _network
 from .strings import __features__ as _strings
 
 
-__all__ = __features__ = _common + _files + _hash + _network + _strings
+__all__ = __features__ = _common + _config + _files + _hash + _network + _strings
