@@ -43,20 +43,20 @@ def __activate(venv_dir):
     site_packages = j(venv_dir, "Lib", "site-packages") if JYTHON else \
                     j(venv_dir, "site-packages") if PYPY else \
                     j(venv_dir, "Lib", "site-packages") if WINDOWS else \
-                    j(venv_dir, "lib", "python{}.{}".format(*sys.version_info),
-                      "site-packages")
+                    j(venv_dir, "lib", "python{}.{}".format(*sys.version_info), "site-packages")
     old = set(__ORIGINAL_SYSPATH)
     site.addsitedir(site_packages)
     new = list(sys.path)
     sys.path = [i for i in new if i not in old] + [i for i in new if i in old]
     sys.real_prefix = __ORIGINAL_SYSRPREFIX or __ORIGINAL_SYSPREFIX
     sys.prefix = venv_dir
+virtualenv.activate = __activate
 
 
 def __check_pip_req_tracker():
     """
-    This checks if the temporary folder of the Pip requirements tracker still
-     exists and corrects the related environment variable accordingly.
+    This checks if the temporary folder of the Pip requirements tracker still exists and corrects the related
+     environment variable accordingly.
     """
     pip_reqt = os.environ.get('PIP_REQ_TRACKER')
     if pip_reqt is not None and not os.path.exists(pip_reqt):
@@ -77,19 +77,18 @@ def __deactivate():
     try:
         # from a normal environment, this key should not exist
         delattr(sys, "real_prefix")
-        # but it is important to also reset it if the script was itself run from
-        #  a virtual environment
+        # but it is important to also reset it if the script was itself run from a virtual environment
         if __ORIGINAL_SYSRPREFIX is not None:
             sys.real_prefix = __ORIGINAL_SYSRPREFIX
     except:
         pass
     __check_pip_req_tracker()
+virtualenv.deactivate = __deactivate
 
 
 def __get_virtualenv(error=True):
     """
-    This gets the currently defined virtual environment or raises an error if no
-     environment is defined.
+    This gets the currently defined virtual environment or raises an error if no environment is defined.
     """
     venv = os.environ.get('VIRTUAL_ENV', "")
     if venv == "" and error:
@@ -107,20 +106,18 @@ def __install(package, *args, **kwargs):
     """
     global pip_proc
     __check_pip_req_tracker()
-    if "-v" in args or "--verbose" in args:
-        verbose = True
-    else:
-        verbose = False
+    verbose = "-v" in args or "--verbose" in args
+    if not verbose:
         args += ("-v", )
     error = kwargs.pop("error", False)
-    cmd = ["install", "-U"] + __parse_args(*args, **kwargs) + [package.strip()]
+    package = package.strip()
+    cmd = ["install", "-U"] + __parse_args(*args, **kwargs) + [package]
     for l in __pip_run(cmd):
         if verbose:
             print(l)
         if not error:
             continue
-        if l.startswith("pip._internal.exceptions") or \
-           l.startswith("DistributionNotFound"):
+        if l.startswith("pip._internal.exceptions") or l.startswith("DistributionNotFound"):
             pip_proc.kill()
             raise PipError(l.split(": ", 1)[1])
         elif l.startswith("Successfully installed"):
@@ -128,6 +125,7 @@ def __install(package, *args, **kwargs):
             if len(m) == 0:
                 raise PipError("Installation of {} failed".format(package))
     return PipPackage(package, error)
+virtualenv.install = __install
 
 
 def __is_installed(package, *args, **kwargs):
@@ -135,22 +133,22 @@ def __is_installed(package, *args, **kwargs):
     This checks if a given package is installed in the virtual environment.
     
     :param package: package name
-    :param args:     options to be used with the pip list command
-    :param kwargs:   keyword-arguments to be used with the pip list command
+    :param args:    options to be used with the pip list command
+    :param kwargs:  keyword-arguments to be used with the pip list command
     """
     found = False
     for name, version in __list_packages(*args, **kwargs):
-        if isinstance(package, string_types) and package == name or \
-           isinstance(package, (list, tuple, set)) and package[0] == name and \
-           package[1] == version:
+        if isinstance(package, string_types) and package == name or isinstance(package, (list, tuple, set)) and \
+           package[0] == name and package[1] == version:
             found = True
+            break
     return found
+virtualenv.is_installed = __is_installed
 
 
 def __list_packages(*args, **kwargs):
     """
-    This lists the packages installed in the currently activated or the given
-     virtual environment.
+    This lists the packages installed in the currently activated or the given virtual environment.
     
     :param venv_dir: virtual environment's directory
     :param args:     options to be used with the pip list command
@@ -158,9 +156,9 @@ def __list_packages(*args, **kwargs):
     """
     cmd = ["list"] + __parse_args(*args, **kwargs)
     for line in __pip_run(cmd, False):
-        if not ("Package" in line and "Version" in line or \
-           "-------" in line or line.strip() == ""):
+        if not ("Package" in line and "Version" in line or "-------" in line or line.strip() == ""):
             yield tuple(_.strip() for _ in line.split(" ", 1))
+virtualenv.list_packages = __list_packages
 
 
 def __parse_args(*args, **kwargs):
@@ -182,8 +180,7 @@ def __parse_args(*args, **kwargs):
 
 def __pip_run(cmd, error=True):
     """
-    This runs a Pip command using the binary from the current virtual
-     environment.
+    This runs a Pip command using the binary from the current virtual environment.
     
     :param cmd: the Pip command and its parameters as a list
     """
@@ -196,15 +193,16 @@ def __pip_run(cmd, error=True):
         yield line[:-1]
 
 
-def __setup(venv_dir, requirements=None, verbose=False):
+def __setup(venv_dir, requirements=None, force_reinstall=False, no_cache=True, verbose=False):
     """
-    This creates (if relevant) and activates a virtual environment. It also
-     allows to define requirements to be installed in this environment.
+    This creates (if relevant) and activates a virtual environment. It also allows to define requirements to be
+     installed in this environment.
     
-    :param venv_dir:     virtual environment's directory
-    :param requirements: list of required package OR path of the requirements
-                          file to be used
-    :param verbose:      displayed Pip output while installing packages
+    :param venv_dir:        virtual environment's directory
+    :param requirements:    list of required package OR path of the requirements file to be used
+    :param force_reinstall: force the reinstallation of required packages
+    :param no_cache:        disable the cache
+    :param verbose:         displayed Pip output while installing packages
     """
     __deactivate()
     venv_dir = os.path.abspath(venv_dir)
@@ -214,12 +212,21 @@ def __setup(venv_dir, requirements=None, verbose=False):
             virtualenv.create_environment(venv_dir)
         except AttributeError:
             virtualenv.cli_run([venv_dir])
+        # other issue: https://github.com/pypa/pipenv/issues/4518
+        #  for Python3, ImportError when using virtualenv installed with pip ;
+        #  works while installed with apt (python3-virtualenv)
     __activate(venv_dir)
     if isinstance(requirements, string_types):
         with open(requirements) as f:
             requirements = [l.strip() for l in f]
     if isinstance(requirements, (tuple, list, set)):
-        args = ["-v"] if verbose else []
+        args = []
+        if force_reinstall:
+            args.append("--force-reinstall")
+        if no_cache:
+            args.append("--no-cache-dir")
+        if verbose:
+            args.append("-v")
         kwargs = {'prefix': venv_dir}
         for req in requirements:
             pkg = __install(req, *args, **kwargs)
@@ -228,12 +235,12 @@ def __setup(venv_dir, requirements=None, verbose=False):
                     raise TopLevelAlreadyExists("{} ({})".format(tl, pkg.name))
                 m = import_module(tl)
                 setattr(virtualenv, tl, m)
+virtualenv.setup = __setup
 
 
 def __teardown(venv_dir=None):
     """
-    This deactivates and removes the given virtual environment or the one
-     defined in the related environment variable.
+    This deactivates and removes the given virtual environment or the one defined in the related environment variable.
 
     :param venv_dir: virtual environment's directory
     """
@@ -241,6 +248,7 @@ def __teardown(venv_dir=None):
     if venv != "":
         __deactivate()
         rmtree(venv, True)
+virtualenv.teardown = __teardown
 
 
 class PipPackage(object):
@@ -264,8 +272,7 @@ class PipPackage(object):
                     continue
             if hasattr(self, "version") or venv == "":
                 break
-            else: # after the first guess, if the package was not found in the
-                  #  virtual environment, deactivate it to retry with main Pip
+            else: # after the first guess, if the package was not found, deactivate it to retry with main Pip
                 globals()['__deactivate']()
         # afterwards, re-enable the virtual environment if relevant
         if venv != "":
@@ -281,6 +288,7 @@ class PipPackage(object):
                 self.top_level = tl.read_lines()
             except IndexError:
                 pass
+virtualenv.PipPackage = PipPackage
 
 
 class VirtualEnv(object):
@@ -288,21 +296,22 @@ class VirtualEnv(object):
     This context manager simplifies the use of a virtual environment.
     
     :param venvdir:      virtual environment's directory
-    :param requirements: list of required package OR path of the requirements
-                          file to be used
-    :param remove:       whether the virtual environment is to be removed after
-                          the entered context
+    :param requirements: list of required package OR path of the requirements file to be used
+    :param reinstall:    force the reinstallation of required packages
+    :param no_cache:     disable the cache
+    :param remove:       whether the virtual environment is to be removed after the entered context
     :param verbose:      displayed Pip output while installing packages
     """
-    def __init__(self, venvdir=None, requirements=None, remove=False,
-                 verbose=False):
+    def __init__(self, venvdir=None, requirements=None, remove=False, reinstall=False, no_cache=False, verbose=False):
+        self.__no_cache = no_cache
+        self.__reinstall = reinstall
         self.__remove = remove
         self.__requirements = requirements
         self.__venv_dir = venvdir or __get_virtualenv()
         self.__verbose = verbose
 
     def __enter__(self):
-        self.setup(self.__venv_dir, self.__requirements, self.__verbose)
+        self.setup(self.__venv_dir, self.__requirements, self.__reinstall, self.__no_cache, self.__verbose)
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
@@ -312,8 +321,8 @@ class VirtualEnv(object):
         try:
             return getattr(virtualenv, name)
         except AttributeError:
-            raise AttributeError("object 'VirtualEnv' has no attribute '{}'"
-                                 .format(name))
+            raise AttributeError("object 'VirtualEnv' has no attribute '{}'".format(name))
+virtualenv.VirtualEnv = VirtualEnv
 
 
 class NotAVirtualEnv(Exception):
@@ -323,13 +332,3 @@ class NotAVirtualEnv(Exception):
 class TopLevelAlreadyExists(Exception):
     pass
 
-
-virtualenv.activate      = __activate
-virtualenv.deactivate    = __deactivate
-virtualenv.install       = __install
-virtualenv.is_installed  = __is_installed
-virtualenv.list_packages = __list_packages
-virtualenv.setup         = __setup
-virtualenv.teardown      = __teardown
-virtualenv.PipPackage    = PipPackage
-virtualenv.VirtualEnv    = VirtualEnv
